/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "IMixCast.h"

#include "MixCastSdkActor.h"
#include "MixCastActorVisibility.h"
#include "Transmission/Thrift/ServiceCommunicator.h"
#include "Transmission/Thrift/ServiceSDKHandler.h"
#include "Transmission/Thrift/ThriftUtil.hpp"

#include "CoreMinimal.h"
#include "CoreGlobals.h"
#include "JsonObjectConverter.h"
#include "Misc/ConfigCacheIni.h"
#include "Modules/ModuleManager.h"

#include <string>

class MIXCAST_API FMixCast : public IMixCast
{
	friend class AMixCastSdkActor;
public:
	FMixCast();

	virtual void StartupModule() override;
	virtual void ShutdownModule() override;

	virtual bool IsActive() { return bActive; }
	void SetActive(bool active);
	FActivationEvent& OnActiveChanged() override { return ActiveChanged; }

	bool SendExperienceEvent(const FString& EventId);
	FReceivedCommandEvent& OnCommandReceived() override { return CommandReceived; }
	FTrackingSpaceChangedEvent& OnTrackingSpaceChanged() override { return TrackingSpaceChanged; }

	DECLARE_EVENT_TwoParams(FServiceSDKHandler, FExternalTextureUpdatedEvent, std::string, SharedTex)
	FExternalTextureUpdatedEvent& OnExternalTextureUpdated() { return ExternalTextureUpdated; }

	FBeforeFrameEvent& OnBeforeFrame() override { return BeforeFrame; }

	FServiceCommunicator* GetClientConnection() { return clientConnection; }

	UWorld* GetWorld() { return activeWorld; }
protected:
	FDelegateHandle ConfigReadyDelegate;

	UPROPERTY() bool bActive;

	FActivationEvent ActiveChanged;
	FReceivedCommandEvent CommandReceived;
	FTrackingSpaceChangedEvent TrackingSpaceChanged;
	FExternalTextureUpdatedEvent ExternalTextureUpdated;
	FBeforeFrameEvent BeforeFrame;

	void RegisterSdkActor(AMixCastSdkActor* actor);
	void UnregisterSdkActor(AMixCastSdkActor* actor);
	TArray<AMixCastSdkActor*> activeSdkActors;
	UWorld* activeWorld;

	FServiceCommunicator* clientConnection = nullptr;
};
