/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "MixCastBPFunctionLibrary.h"
#include "IMixCast.h"
#include "MixCastSdkActor.h"
#include "MixCastLog.h"

void UMixCastBPFunctionLibrary::SendExperienceEvent(FString eventStr)
{
	if (!IMixCast::IsAvailable())
		return;

	IMixCast* mod = &IMixCast::Get();
	if (mod->IsActive()) {
		mod->SendExperienceEvent(eventStr);
	}
}

void UMixCastBPFunctionLibrary::SetActorVisibilityForMixCast(AActor* actor, 
	bool showForRegularCameras,
	bool showForMixCastCameras,
	TEnumAsByte<EMixCastCameraMode::Type> mixCastCameraMode,
	TEnumAsByte<ERenderTypeCondition::Type> renderTypeCondition,
	TEnumAsByte<EPerspectiveCondition::Type> perspectiveCondition)
{
	if (!actor)
	{
		MIXCAST_LOG("Error: Called SetActorVisibilityForMixCast without specifying the actor!");
		return;
	}

	UMixCastActorVisibility* visibility = Cast<UMixCastActorVisibility>(actor->GetComponentByClass(UMixCastActorVisibility::StaticClass()));
	if (!visibility)
		visibility = NewObject<UMixCastActorVisibility>(actor);
	else
		visibility->UnregisterComponent();

	visibility->showForRegularCameras = showForRegularCameras;
	visibility->showForMixCastCameras = showForMixCastCameras;
	visibility->mixCastCameraMode = mixCastCameraMode;
	visibility->renderTypeCondition = renderTypeCondition;
	visibility->perspectiveCondition = perspectiveCondition;

	visibility->RegisterComponent();
}