/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "MixCastActorVisibility.h"
#include "MixCastSdkActor.h"
#include "MixCast.h"
#include "Data/SdkData.h"
#include "Rendering/ExpCamera.h"
#include "Transmission/Generated/VirtualCamera_types.h"

#include "Engine/Engine.h"
#include "GameFramework/PlayerController.h"

UMixCastActorVisibility::UMixCastActorVisibility()
{

}

void UMixCastActorVisibility::BeginPlay()
{
	Super::BeginPlay();

	if (!this->showForRegularCameras) {
		APlayerController* player = GEngine->GetFirstLocalPlayerController(GetWorld());
		if (player)
			player->HiddenActors.Add(GetOwner());
	}

	FrameStartedDelegate = AExpCamera::OnFrameStarted().AddUObject(this, &UMixCastActorVisibility::HandleFrameStarted);
}
void UMixCastActorVisibility::EndPlay(const EEndPlayReason::Type endPlayReason)
{
	Super::EndPlay(endPlayReason);

	if (!this->showForRegularCameras) {
		APlayerController* player = GEngine->GetFirstLocalPlayerController(GetWorld());
		if (player)
			player->HiddenActors.Remove(GetOwner());
	}

	AExpCamera::OnFrameStarted().Remove(FrameStartedDelegate);
}

void UMixCastActorVisibility::HandleFrameStarted(AExpCamera* cam, VirtualCamera* camInfo)
{
	if (!ShouldShowToCamera(camInfo))
		cam->SetActorHidden(GetOwner());
}

bool UMixCastActorVisibility::ShouldShowToCamera(VirtualCamera* camInfo)
{
	bool showForThisMixCastCamera = showForMixCastCameras;
	switch (mixCastCameraMode.GetValue())
	{
	case EMixCastCameraMode::Always:
		break;
	case EMixCastCameraMode::IfAnyConditionMet:
		if (!HasAnyFlag(camInfo))
			showForThisMixCastCamera = !showForThisMixCastCamera;
		break;
	case EMixCastCameraMode::IfAllConditionsMet:
		if (!HasAllFlags(camInfo))
			showForThisMixCastCamera = !showForThisMixCastCamera;
		break;
	}
	return showForThisMixCastCamera;
}
bool UMixCastActorVisibility::HasAnyFlag(VirtualCamera* camInfo)
{
	switch (renderTypeCondition.GetValue())
	{
	case ERenderTypeCondition::Mixed:
		if (camInfo->usesForeground)
			return true;
		break;
	case ERenderTypeCondition::Virtual:
		if (!camInfo->usesForeground)
			return true;
		break;
	}

	switch (backdropStyleCondition.GetValue())
	{
	case EBackdropStyleCondition::Virtual:
		if (!camInfo->isBackgroundTranslucent)
			return true;
		break;
	case EBackdropStyleCondition::Physical:
		if (camInfo->isBackgroundTranslucent)
			return true;
		break;
	}

	switch (perspectiveCondition.GetValue())
	{
	case EPerspectiveCondition::First:
		if (camInfo->isFirstPerson)
			return true;
		break;
	case EPerspectiveCondition::Third:
		if (!camInfo->isFirstPerson)
			return true;
		break;
	}

	return false;
}
bool UMixCastActorVisibility::HasAllFlags(mixcast::data::VirtualCamera* camInfo)
{
	switch (renderTypeCondition.GetValue())
	{
	case ERenderTypeCondition::Mixed:
		if (!camInfo->usesForeground)
			return false;
		break;
	case ERenderTypeCondition::Virtual:
		if (camInfo->usesForeground)
			return false;
		break;
	}

	switch (backdropStyleCondition.GetValue())
	{
	case EBackdropStyleCondition::Virtual:
		if (camInfo->isBackgroundTranslucent)
			return false;
		break;
	case EBackdropStyleCondition::Physical:
		if (!camInfo->isBackgroundTranslucent)
			return false;
		break;
	}

	switch (perspectiveCondition.GetValue())
	{
	case EPerspectiveCondition::First:
		if (!camInfo->isFirstPerson)
			return false;
		break;
	case EPerspectiveCondition::Third:
		if (camInfo->isFirstPerson)
			return false;
		break;
	}

	return true;
}