/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "Engine/CanvasRenderTarget2D.h"
#include "Materials/MaterialInstanceDynamic.h"

#include "MixCastFrameCanvas.generated.h"

class AExpCamera;

UENUM()
namespace EMaterialAlphaType
{
	enum Type
	{
		Opaque = 0,
		Keyed = 1,
		ForceAdditive = 2,
		AlphaChannel = 3,
	};
}

UCLASS()
class MIXCAST_API UMixCastFrameCanvas : public UCanvasRenderTarget2D
{
	GENERATED_UCLASS_BODY()

public:
	static UMixCastFrameCanvas* Create(AExpCamera* source, int layerWidth, int layerHeight);

	UMixCastFrameCanvas();
	~UMixCastFrameCanvas();

	int GetLayerWidth() { return LayerWidth; }
	int GetLayerHeight() { return LayerHeight; }

	UTextureRenderTarget2D* GetFullTex() { return fullRenderTex; }
	UTextureRenderTarget2D* GetFullAlphaTex() { return fullAlphaTex; }
	UTextureRenderTarget2D* GetForegroundTex() { return foregroundTex; }
	UTextureRenderTarget2D* GetForegroundAlphaTex() { return foregroundAlphaTex; }

protected:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* fullRenderTex;
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* fullAlphaTex;
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* foregroundTex;
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* foregroundAlphaTex;

	UFUNCTION()
	void BlitLayers(UCanvas* Canvas, int32 Width, int32 Height);

	AExpCamera* Source;
	int LayerWidth, LayerHeight;

	void Initialize(AExpCamera* source, int width, int height);
	UTextureRenderTarget2D* CreateRenderTarget(int width, int height);

private:
	UPROPERTY(VisibleAnywhere, Category = "Materials")
		TArray<UMaterialInstanceDynamic*> fullRenderMats;
	UPROPERTY(VisibleAnywhere, Category = "Materials")
		TArray<UMaterialInstanceDynamic*> foregroundMats;

	UMaterialInstanceDynamic* GetTransferMaterial(TArray<UMaterialInstanceDynamic*>& mats, bool isTransparent);
};