#pragma once

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <Windows.h>
#include <stdio.h>
#include <conio.h>
#include <d3d11.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#include "SharedSurfaceWriter.hpp"

struct SharedSurfaceProducerDesc
{
	int chainLength;
};

class SharedSurfaceProducer
{
public:
	SharedSurfaceProducer(LPCWSTR chainId, int chainLength)
	{
		LPWSTR key = new TCHAR[256];
		//Metadata
		wsprintf(key, TEXT("MixCast-Chain(%s)-Desc"), chainId);
		descFile = CreateFileMapping(
			INVALID_HANDLE_VALUE,
			NULL,
			PAGE_READWRITE,
			0,							// (high-order DWORD)
			sizeof(SharedSurfaceProducerDesc),  // (low-order DWORD)
			key);						// name of mapping object
		SharedSurfaceProducerDesc* descView = (SharedSurfaceProducerDesc*)MapViewOfFile(descFile,
			FILE_MAP_ALL_ACCESS,
			0,
			0,
			sizeof(SharedSurfaceProducerDesc));
		descView->chainLength = chainLength;
		UnmapViewOfFile(descView);

		for (int i = 0; i < chainLength; i++)
		{
			wsprintf(key, TEXT("%s[%d]"), chainId, i);
			surfaces.Add(new SharedSurfaceWriter(key));
		}

		delete[] key;
	}
	~SharedSurfaceProducer()
	{
		for (int i = 0; i < surfaces.Num(); i++)
			delete surfaces[i];
		surfaces.Reset();

		CloseHandle(descFile);
	}

	bool TryPushFrame(ID3D11Texture2D* srcTex, unsigned long long newSyncTime, unsigned long long newRenderTime, unsigned long newFrameIndex, int timeout)
	{
		if (!WaitUntilReadyForNextWrite(timeout))
			return false;

		SharedSurfaceWriter* nextSurface = GetNextWriteSurface();
		nextSurface->Update(srcTex, newSyncTime, newRenderTime, newFrameIndex);
		if (srcTex != NULL)
			nextSurface->EnqueueTextureCopy();
		writeCount++;
		return true;
	}
	bool PushEndOfFile(int timeout)
	{
		if (!WaitUntilReadyForNextWrite(timeout))
			return false;

		GetNextWriteSurface()->SignalEndOfStream();
		return true;
	}

	void EnqueueSurfaceFenceChecks()
	{
		for (int i = 0; i < surfaces.Num(); i++)
			surfaces[i]->EnqueueFenceCheck();
	}

private:
	HANDLE descFile = NULL;

	TArray<SharedSurfaceWriter*> surfaces;
	unsigned long writeCount = 0;

	SharedSurfaceWriter* GetNextWriteSurface()
	{
		return surfaces[writeCount % surfaces.Num()];
	}

	bool WaitUntilReadyForNextWrite(int timeout)
	{
		return GetNextWriteSurface()->WaitForReadyToWrite(timeout);
	}
};

