/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "ExpViewfinderSpawner.h"

#include "MixCast.h"
#include "Data/SdkData.h"
#include "MixCastLog.h"

#include "Async/Async.h"

#include "Camera/CameraComponent.h"
#include "Engine/Engine.h"
#include "Engine/World.h"
#include "GameFramework/PlayerController.h"
#include "HeadMountedDisplay.h"
#include "Kismet/GameplayStatics.h"
#include "Runtime/Launch/Resources/Version.h"
#include "HeadMountedDisplay.h"

UExpViewfinderSpawner::UExpViewfinderSpawner(const FObjectInitializer &ObjectInitializer)
	: Super(ObjectInitializer)
{
}

void UExpViewfinderSpawner::SynchronizeSpawnedViewfinders()
{
	for (int i = viewfinders.Num() - 1; i >= 0; i--)
	{
		bool foundAgain = false;
		for (int j = 0; j < SdkData::Viewfinders.size() && !foundAgain; j++)
			foundAgain |= SdkData::Viewfinders[j].identifier.compare(viewfinders[i]->Identifier) == 0;

		if (!foundAgain)
		{
			if (viewfinders[i]->IsValidLowLevel())
				viewfinders[i]->Destroy();
			viewfinders.RemoveAt(i);
		}
	}

	for (int i = 0; i < SdkData::Viewfinders.size(); i++)
	{
		bool alreadyFound = false;
		for (int j = 0; j < viewfinders.Num() && !alreadyFound; j++)
			alreadyFound |= viewfinders[j]->Identifier.compare(SdkData::Viewfinders[i].identifier) == 0;

		if (!alreadyFound)
		{
			UClass* viewfinderClass = SdkData::GetProjectSettings()->ViewfinderClass.TryLoadClass<AExpViewfinder>();
			if (viewfinderClass == nullptr)
				viewfinderClass = AExpViewfinder::StaticClass();
			AActor* viewfinderActor = UGameplayStatics::BeginDeferredActorSpawnFromClass(this, viewfinderClass, FTransform::Identity);
			AExpViewfinder* viewfinder = Cast<AExpViewfinder>(viewfinderActor);
			viewfinder->Identifier = SdkData::Viewfinders[i].identifier;
			viewfinder->GetRootComponent()->AttachToComponent(this, FAttachmentTransformRules::KeepRelativeTransform);
			UGameplayStatics::FinishSpawningActor(viewfinderActor, FTransform::Identity);
			viewfinders.Add(viewfinder);
		}
	}
}

void UExpViewfinderSpawner::ClearSpawnedViewfinders()
{
	for (int i = 0; i < viewfinders.Num(); i++)
	{
		if (viewfinders[i]->IsValidLowLevel())
			viewfinders[i]->Destroy();
	}
	viewfinders.Empty();
}